﻿using System;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Windows.Media.Imaging;

namespace SampleApp
{
    /// <summary>
    /// 表示单个图像的相关信息
    /// </summary>
    public class PhotoData : INotifyPropertyChanged
    {
        #region 私有字段
        int width = default(int), height = default(int);
        double dpix = default(double), dpiy = default(double);
        BitmapImage bitmap = null;
        double area;
        #endregion

        #region INotifyPropertyChanged成员
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion

        #region 构造函数
        public PhotoData(string filePath)
        {
            // 实例化图像对象
            PhotoImage = new BitmapImage();
            PhotoImage.DecodePixelWidth = 100;
            // 初始化图像
            PhotoImage.BeginInit();
            PhotoImage.UriSource = new Uri(filePath);
            PhotoImage.EndInit();

            // 获取需要的参数
            Width = PhotoImage.PixelWidth;
            Height = PhotoImage.PixelHeight;
            DpiX = PhotoImage.DpiX;
            DpiY = PhotoImage.DpiY;
            // 计算面积
            ComputeArea();
        }
        #endregion

        #region 方法
        private void OnPropertyChanged([CallerMemberName]string prpname = "")
        {
            this.PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(prpname));
        }

        /// <summary>
        /// 计算面积
        /// </summary>
        private void ComputeArea()
        {
            // 先将宽度和高度转为英寸
            double inchW = Width / DpiX;
            double inchH = Height / DpiY;
            /*
                面积单位为平方米
                1英寸 = 2.54厘米
            */
            Area = (inchW * 2.54d) * (inchH * 2.54d) / 10000d;
        }
        #endregion

        #region 属性
        /// <summary>
        /// 照片宽度
        /// </summary>
        public int Width
        {
            get { return width; }
            private set
            {
                if (value != width)
                {
                    width = value;
                    OnPropertyChanged();
                }
            }
        }

        /// <summary>
        /// 照片高度
        /// </summary>
        public int Height
        {
            get { return height; }
            private set
            {
                if (value != height)
                {
                    height = value;
                    OnPropertyChanged();
                }
            }
        }

        /// <summary>
        /// 水平分辨率
        /// </summary>
        public double DpiX
        {
            get { return dpix; }
            private set
            {
                if (dpix != value)
                {
                    dpix = value;
                    OnPropertyChanged();
                }
            }
        }

        /// <summary>
        /// 垂直分辨率
        /// </summary>
        public double DpiY
        {
            get { return dpiy; }
            private set
            {
                if (value != dpiy)
                {
                    dpiy = value;
                    OnPropertyChanged();
                }
            }
        }

        /// <summary>
        /// 图像实例
        /// </summary>
        public BitmapImage PhotoImage
        {
            get { return bitmap; }
            private set
            {
                if (bitmap != value)
                {
                    bitmap = value;
                    OnPropertyChanged();
                }
            }
        }

        /// <summary>
        /// 面积（平方米）
        /// </summary>
        public double Area
        {
            get { return area; }
            private set
            {
                if (value != area)
                {
                    area = value;
                    OnPropertyChanged();
                }
            }
        }
        #endregion
    }
}
